# -*- coding: utf-8 -*-
"""
    django_geopostcodes.tests.test_helpers
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Test :py:module:`django_geopostcodes.helpers`.
"""
from __future__ import absolute_import, print_function, unicode_literals
import os

from django.test import TestCase
from django_geopostcodes.helpers import import_localities
from django_geopostcodes.models import Locality


def create_sample_localities():
    import_localities(os.path.join(os.path.dirname(__file__),
                                   'fixtures',
                                   'Sample_GeoPC_AU_Places.csv'),
                      "\t")


class ImportLocalitiesTestCase(TestCase):

    def test_import_localities(self):
        self.assertEqual(Locality.objects.count(), 0)
        create_sample_localities()
        self.assertEqual(Locality.objects.count(), 100)



# -*- coding: utf-8 -*-
"""
    tests.testapp.settings
    ~~~~~~~~~~~~~~~~~~~~~~

    Django settings for tests.
"""
import os

TEST_RUNNER = 'django.test.runner.DiscoverRunner'

# Django replaces this, but it still wants it. *shrugs*
DATABASES = {
    'default': {
        'ENGINE': 'django.contrib.gis.db.backends.postgis',
        'NAME': os.environ.get('TEST_DB_NAME', 'django_geopostcodes_test'),
    }
}

if os.environ.get('TEST_DB_USER', None):
    DATABASES['default']['USER'] = os.environ.get('TEST_DB_USER')

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.admin',
    'django.contrib.sessions',
    'django.contrib.gis',
    'django.contrib.messages',
    'django_geopostcodes',
)

MIDDLEWARE_CLASSES = {}

SECRET_KEY = '53cr3773rc3553cr3773rc3553cr3773rc3553cr3773rc35'

try:
    GDAL_LIBRARY_PATH = os.environ['GDAL_LIBRARY_PATH']
except KeyError:
    pass

try:
    GEOS_LIBRARY_PATH = os.environ['GEOS_LIBRARY_PATH']
except KeyError:
    pass

try:
    SPATIALITE_LIBRARY_PATH = os.environ['SPATIALITE_LIBRARY_PATH']
except KeyError:
    pass

MIDDLEWARE = [
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
]

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.contrib.messages.context_processors.messages",
                "django.template.context_processors.request",
            ]
        },
    }
]

DEFAULT_AUTO_FIELD = 'django.db.models.BigAutoField'

USE_TZ = True



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.managers
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Django model managers for django-geopostcodes.
"""
from __future__ import absolute_import, print_function, unicode_literals

from django.db.models import QuerySet
from django.db.models.manager import Manager
from django.contrib.gis.db import models


class LocalityQuerySet(QuerySet):
    """
    Locality QuerySet.
    """
    anything_fields = ('country',
                       'region1', 'region2', 'region3', 'region4',
                       'locality', 'postcode', 'suburb')

    def anything(self, lookup_type, value, fields=anything_fields):
        queries = [models.Q(**{'%s__%s' % (field, lookup_type): value}) for field in fields]

        # Take one Q object from the list
        query = queries.pop()

        # Or the Q object with the ones remaining in the list
        for item in queries:
            query |= item

        return self.filter(query)

    def anything_icontains(self, value, fields=anything_fields):
        return self.anything('icontains', value, fields)

    def anything_contains(self, value, fields=anything_fields):
        return self.anything('contains', value, fields)

    def anything_exact(self, value, fields=anything_fields):
        return self.anything('exact', value, fields)

    def anything_iexact(self, value, fields=anything_fields):
        return self.anything('iexact', value, fields)

    def anything_startswith(self, value, fields=anything_fields):
        return self.anything('startswith', value, fields)

    def anything_istartswith(self, value, fields=anything_fields):
        return self.anything('istartswith', value, fields)

    def anything_endswith(self, value, fields=anything_fields):
        return self.anything('endswith', value, fields)

    def anything_iendswith(self, value, fields=anything_fields):
        return self.anything('iendswith', value, fields)


class LocalityManager(Manager.from_queryset(LocalityQuerySet)):
    "Overrides Manager to return Geographic QuerySets."

    # This manager should be used for queries on related fields
    # so that geometry columns on Oracle and MySQL are selected
    # properly.
    use_for_related_fields = True



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.admin
    ~~~~~~~~~~~~~~~~~~~~~~~~

    Django admin definition for django-geopostcodes.

"""
from django.contrib.gis import admin
from .models import Locality


class LocalityAdmin(admin.GISModelAdmin):
    list_display = ['id', 'iso', 'country', 'language', 'region1', 'region2',
                    'region3', 'region4', 'locality', 'postcode', 'suburb',
                    'latitude', 'longitude', 'elevation', 'iso2', 'fips',
                    'nuts', 'hasc', 'stat', 'timezone', 'utc', 'dst']
    list_filter = ['iso',
                   'country',
                   'language',
                   'timezone',
                   'utc',
                   'dst']
    search_fields = ['iso', 'country', 'language', 'region1', 'region2',
                     'region3', 'region4', 'locality', 'postcode', 'suburb',
                     'latitude', 'longitude', 'elevation', 'iso2', 'fips',
                     'nuts', 'hasc', 'stat', 'timezone', 'utc', 'dst']


admin.site.register(Locality, LocalityAdmin)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django_countries.fields
import django.contrib.gis.db.models.fields


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Locality',
            fields=[
                ('id', models.AutoField(auto_created=True, verbose_name='ID', primary_key=True, serialize=False)),
                ('iso', django_countries.fields.CountryField(help_text='ISO 3166-1 Country code', verbose_name='iso', max_length=2, db_index=True)),
                ('country', models.CharField(verbose_name='country', max_length=50, help_text='Country name')),
                ('language', models.CharField(help_text='ISO 639-1 language code', verbose_name='language', max_length=2, db_index=True)),
                ('region1', models.CharField(help_text='Administrative region level 1', verbose_name='region 1', max_length=80, db_index=True)),
                ('region2', models.CharField(help_text='Administrative region level 2', verbose_name='region 2', max_length=80, db_index=True)),
                ('region3', models.CharField(help_text='Administrative region level 3', verbose_name='region 3', max_length=80, db_index=True)),
                ('region4', models.CharField(help_text='Administrative region level 4', verbose_name='region 4', max_length=80, db_index=True)),
                ('locality', models.CharField(help_text='Locality name', verbose_name='locality', max_length=80, db_index=True)),
                ('postcode', models.CharField(help_text='ZIP/Postal code', verbose_name='locality', max_length=15, db_index=True)),
                ('suburb', models.CharField(verbose_name='locality', max_length=80, help_text='Locality subdivision')),
                ('latitude', models.FloatField(verbose_name='latitude', help_text='Latitude coordinates in WGS84')),
                ('longitude', models.FloatField(verbose_name='longitude', help_text='Longitude coordinates in WGS84')),
                ('elevation', models.IntegerField(verbose_name='elevation', help_text='Elevation in meters')),
                ('point', django.contrib.gis.db.models.fields.PointField(srid=4326, db_index=True)),
                ('iso2', models.CharField(verbose_name='ISO2', max_length=10, help_text='ISO 3166-2 Region code')),
                ('fips', models.CharField(verbose_name='FIPS', max_length=10, help_text='NGA Geopolitical code (formerly FIPS PUB 10-4)')),
                ('nuts', models.CharField(verbose_name='NUTS', max_length=12, help_text='European subdivision code')),
                ('hasc', models.CharField(verbose_name='HASC', max_length=12, help_text='Hierarchical administrative subdivision code')),
                ('stat', models.CharField(verbose_name='STAT', max_length=20, help_text='National statistics/census code')),
                ('timezone', models.CharField(verbose_name='timezone', max_length=30, help_text='Time zone name (Olson)')),
                ('utc', models.CharField(verbose_name='utc', max_length=10, help_text='Coordinated Universal Time')),
                ('dst', models.CharField(verbose_name='dst', max_length=10, help_text='Daylight Saving Time')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_geopostcodes', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='locality',
            options={'verbose_name_plural': 'localities', 'verbose_name': 'locality'},
        ),
        migrations.AlterField(
            model_name='locality',
            name='postcode',
            field=models.CharField(db_index=True, verbose_name='postcode', help_text='ZIP/Postal code', max_length=15),
        ),
        migrations.AlterField(
            model_name='locality',
            name='suburb',
            field=models.CharField(verbose_name='suburb', help_text='Locality subdivision', max_length=80),
        ),
    ]



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.tests.test_models
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Tests for django-geopostcodes models.
"""
from __future__ import absolute_import, print_function, unicode_literals
import os
import unittest

import pytest
from django.contrib.gis.geos import Point
from django.contrib.gis.measure import D

from django_geopostcodes.helpers import import_localities
from django_geopostcodes.models import Locality


@pytest.mark.django_db()
class LocalityQuerySetTestCase(unittest.TestCase):

    def setUp(self):
        import_localities(os.path.join(os.path.dirname(__file__),
                                       'fixtures',
                                       'Sample_GeoPC_AU_Places.csv'),
                          "\t")

    def test_count(self):
        self.assertEqual(Locality.objects.count(), 100)

    def test_point_equal(self):
        self.assertEqual(Locality.objects.filter(point=Point(149.075273, -33.648855)).count(), 1)

    def test_distance_one_cm(self):
        self.assertEqual(Locality.objects.filter(point__distance_lte=(Point(149.075273, -33.648855), D(cm=1))).count(), 1)

    def test_distance_one_km(self):
        self.assertEqual(Locality.objects.filter(point__distance_lte=(Point(149.075273, -33.648855), D(cm=1))).count(), 1)

    def test_distance_one_hundred_km(self):
        self.assertEqual(Locality.objects.filter(point__distance_lte=(Point(149.075273, -33.648855), D(km=20))).count(), 5)
        print(
            Locality.objects.filter(point__distance_lte=(Point(149.075273, -33.648855), D(km=20)))
        )

    def test_anything_contains(self):
        self.assertEqual(Locality.objects.count(), 100)
        self.assertEqual(Locality.objects.anything_contains('Australia').count(), 100)
        self.assertEqual(Locality.objects.anything_contains('New South').count(), 100)
        self.assertEqual(Locality.objects.anything_contains('Victoria').count(), 0)
        self.assertEqual(Locality.objects.anything_contains('Blayney').count(), 7)
        self.assertEqual(Locality.objects.anything_contains('Panuara').count(), 1)
        self.assertEqual(Locality.objects.anything_contains('Cowra').count(), 9)
        self.assertEqual(Locality.objects.anything_contains('2866').count(), 5)

    def test_anything_icontains(self):
        self.assertEqual(Locality.objects.count(), 100)
        self.assertEqual(Locality.objects.anything_icontains('Austr').count(), 100)
        self.assertEqual(Locality.objects.anything_icontains('New South Wales').count(), 100)
        self.assertEqual(Locality.objects.anything_icontains('Vic').count(), 0)
        self.assertEqual(Locality.objects.anything_icontains('Blayney').count(), 7)
        self.assertEqual(Locality.objects.anything_icontains('Panu').count(), 1)
        self.assertEqual(Locality.objects.anything_icontains('Cowr').count(), 9)
        self.assertEqual(Locality.objects.anything_icontains('286').count(), 17)
        self.assertEqual(Locality.objects.anything_icontains('28').count(), 76)

    def test_anything_exact(self):
        self.assertEqual(Locality.objects.count(), 100)
        self.assertEqual(Locality.objects.anything_exact('Australia').count(), 100)
        self.assertEqual(Locality.objects.anything_exact('New South').count(), 0)
        self.assertEqual(Locality.objects.anything_exact('Victoria').count(), 0)
        self.assertEqual(Locality.objects.anything_exact('Blayney').count(), 7)
        self.assertEqual(Locality.objects.anything_exact('Panuara').count(), 1)
        self.assertEqual(Locality.objects.anything_exact('Cowra').count(), 9)
        self.assertEqual(Locality.objects.anything_exact('Cowr').count(), 0)
        self.assertEqual(Locality.objects.anything_exact('2866').count(), 5)
        self.assertEqual(Locality.objects.anything_exact('28').count(), 0)

    def test_anything_iexact(self):
        self.assertEqual(Locality.objects.count(), 100)
        self.assertEqual(Locality.objects.anything_iexact('Australia').count(), 100)
        self.assertEqual(Locality.objects.anything_iexact('New South').count(), 0)
        self.assertEqual(Locality.objects.anything_iexact('Victoria').count(), 0)
        self.assertEqual(Locality.objects.anything_iexact('Blayney').count(), 7)
        self.assertEqual(Locality.objects.anything_iexact('Panuara').count(), 1)
        self.assertEqual(Locality.objects.anything_iexact('Cowra').count(), 9)
        self.assertEqual(Locality.objects.anything_iexact('Cowr').count(), 0)
        self.assertEqual(Locality.objects.anything_iexact('2866').count(), 5)
        self.assertEqual(Locality.objects.anything_iexact('28').count(), 0)

    def test_anything_startswith(self):
        self.assertEqual(Locality.objects.count(), 100)
        self.assertEqual(Locality.objects.anything_startswith('Aus').count(), 100)
        self.assertEqual(Locality.objects.anything_startswith('ustralia').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('New South').count(), 100)
        self.assertEqual(Locality.objects.anything_startswith('Victoria').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('Blayn').count(), 7)
        self.assertEqual(Locality.objects.anything_startswith('ayney').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('Panuara').count(), 1)
        self.assertEqual(Locality.objects.anything_startswith('Cowra').count(), 9)
        self.assertEqual(Locality.objects.anything_startswith('Cowr').count(), 9)
        self.assertEqual(Locality.objects.anything_startswith('owra').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('2866').count(), 5)
        self.assertEqual(Locality.objects.anything_startswith('66').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('28').count(), 76)

    def test_anything_istartswith(self):
        self.assertEqual(Locality.objects.count(), 100)
        self.assertEqual(Locality.objects.anything_startswith('Aus').count(), 100)
        self.assertEqual(Locality.objects.anything_startswith('ustralia').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('New South').count(), 100)
        self.assertEqual(Locality.objects.anything_startswith('Victoria').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('Blayn').count(), 7)
        self.assertEqual(Locality.objects.anything_startswith('ayney').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('Panuara').count(), 1)
        self.assertEqual(Locality.objects.anything_startswith('Cowra').count(), 9)
        self.assertEqual(Locality.objects.anything_startswith('Cowr').count(), 9)
        self.assertEqual(Locality.objects.anything_startswith('owra').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('2866').count(), 5)
        self.assertEqual(Locality.objects.anything_startswith('66').count(), 0)
        self.assertEqual(Locality.objects.anything_startswith('28').count(), 76)

    def test_chain(self):
        self.assertEqual(
            Locality.objects.anything_icontains('Mandurama').filter(point__distance_lte=(Point(149.075273, -33.648855), 0.00001)).count(),
            1
        )
        self.assertEqual(
            Locality.objects.anything_icontains('New South').filter(point__distance_lte=(Point(149.075273, -33.648855), 0.00001)).count(),
            1
        )
        self.assertEqual(
            Locality.objects.anything_icontains('Panuara').filter(point__distance_lte=(Point(149.075273, -33.648855), 0.00001)).count(),
            0
        )



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.models
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    Django models for django-geopostcodes.

"""
from __future__ import absolute_import, print_function, unicode_literals
from django.contrib.gis.db import models
from django.utils.translation import gettext_lazy as _
from django_countries.fields import CountryField
from .managers import LocalityManager


class Locality(models.Model):
    """
    Defines an Locality as defined in GeoPostCodes GeoPC_xx_Places.csv files.
    """
    iso = CountryField(_('iso'), db_index=True, help_text=_('ISO 3166-1 Country code'))
    country = models.CharField(_('country'), max_length=50, help_text=_('Country name'))
    language = models.CharField(_('language'), max_length=2, db_index=True, help_text=_('ISO 639-1 language code'))
    region1 = models.CharField(_('region 1'), max_length=80, db_index=True, help_text=_('Administrative region level 1'))
    region2 = models.CharField(_('region 2'), max_length=80, db_index=True, help_text=_('Administrative region level 2'))
    region3 = models.CharField(_('region 3'), max_length=80, db_index=True, help_text=_('Administrative region level 3'))
    region4 = models.CharField(_('region 4'), max_length=80, db_index=True, help_text=_('Administrative region level 4'))
    locality = models.CharField(_('locality'), max_length=80, db_index=True, help_text=_('Locality name'))
    postcode = models.CharField(_('postcode'), max_length=15, db_index=True, help_text=_('ZIP/Postal code'))
    suburb = models.CharField(_('suburb'), max_length=80, help_text=_('Locality subdivision'))
    latitude = models.FloatField(_('latitude'), help_text=_('Latitude coordinates in WGS84'))
    longitude = models.FloatField(_('longitude'), help_text=_('Longitude coordinates in WGS84'))
    elevation = models.IntegerField(_('elevation'), help_text=_('Elevation in meters'))
    point = models.PointField(db_index=True)
    iso2 = models.CharField(_('ISO2'), max_length=10, help_text=_('ISO 3166-2 Region code'))
    fips = models.CharField(_('FIPS'), max_length=10, help_text=_('NGA Geopolitical code (formerly FIPS PUB 10-4)'))
    nuts = models.CharField(_('NUTS'), max_length=12, help_text=_('European subdivision code'))
    hasc = models.CharField(_('HASC'), max_length=12, help_text=_('Hierarchical administrative subdivision code'))
    stat = models.CharField(_('STAT'), max_length=20, help_text=_('National statistics/census code'))
    timezone = models.CharField(_('timezone'), max_length=30, help_text=_('Time zone name (Olson)'))
    utc = models.CharField(_('utc'), max_length=10, help_text=_('Coordinated Universal Time'))
    dst = models.CharField(_('dst'), max_length=10, help_text=_('Daylight Saving Time'))

    objects = LocalityManager()

    class Meta:
        verbose_name = _('locality')
        verbose_name_plural = _('localities')

    def __str__(self):
        return self.locality



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.management.commands.import_localities
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Import a GeoPostCodes Locality file.
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging

from django.core.management.base import BaseCommand
from django_geopostcodes.helpers import import_localities
from django.utils.translation import gettext_lazy as _


logger = logging.getLogger('console.' + __name__)


class Command(BaseCommand):
    """
    Import a GeoPostCodes Locality file.
    """
    help = _("Import a GeoPostCodes Places.csv file.")

    def add_arguments(self, parser):
        parser.add_argument('infile', type=str, help=_("Path to CSV file."))
        parser.add_argument('--delimiter', type=str, default=';', help=_("Set the CSV delimiter."))

    def handle(self, *args, **options):
        import_localities(options['infile'], options['delimiter'])



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.apps
    ~~~~~~~~~~~~~~~~~~~~~~~~

    Django app definition for django-geopostcodes.
"""
from __future__ import absolute_import, print_function, unicode_literals
from django.apps import AppConfig


class DjangoGeoPostCodesAppConfig(AppConfig):
    name = 'django_geopostcodes'
    verbose_name = 'Django GeoPostCodes'
    default_auto_field = 'django.db.models.AutoField'



#!/usr/bin/env python

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Django>=2.2',
    'django_countries>=7.3.2',
]

test_requirements = [
    'pytest>=7.1.2',
    'pytest-django>=4.5.2'
]

setup(
    author="Alex Hayes",
    author_email='alex@alution.com',
    python_requires='>=3.6',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.2',
        'Framework :: Django :: 4.0',
    ],
    description="Django modelling for geopostcodes.com",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/x-rst',
    include_package_data=True,
    keywords='django_geopostcodes',
    name='django-geopostcodes',
    packages=find_packages(include=['django_geopostcodes', 'django_geopostcodes.*']),
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/alexhayes/django_geopostcodes',
    version='0.3.0',
    zip_safe=False,
)



# -*- coding: utf-8 -*-
"""
    django_geopostcodes.helpers
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Helper functions for django-geopostcodes.
"""
from __future__ import absolute_import, print_function, unicode_literals

import csv

from django.contrib.gis.geos import Point
from django.db.transaction import atomic

from .models import Locality


def import_localities(path, delimiter=';'):
    """
    Import localities from a CSV file.

    :param path: Path to the CSV file containing the localities.
    """

    creates = []
    updates = []

    with open(path, mode="r") as infile:
        reader = csv.DictReader(infile, delimiter=str(delimiter))

        with atomic():
            for row in reader:
                row['point'] = Point(float(row['longitude']),
                                     float(row['latitude']))
                locality, created = Locality.objects.update_or_create(
                    id=row['id'],
                    defaults=row
                )
                if created:
                    creates.append(locality)
                else:
                    updates.append(locality)

    return creates, updates
